function [MLvec,coverage]=MLMGD(etaMat,lnPL)
% PURPOSE: Compute the Modified Harmonic Mean estimate of the Marginal Lik.
% -----------------------------------------------------
% USAGE: [MLvec,coverage]=MLMGD(etaMat,lnPL)
% where the inputs are
% etaMat = (M x k) matrix where ith row is the ith draw from the MCMC
%     simulator on the transformed parameter vector, i.e. that one that has  
%     already been transformed to have unrestricted domain.
% lnPL = (M x 1) vector with log posterior associated to each parameter draw.
%     (M = number MCMC draws and k = number of parameters)
% 
% Note that the log posterior = log prior + log likelihood and prior  
% need include also normalizing constants (not only kernel).
% 
% RETURNS:
% MLvec = the 9 x 1 vector with the 9 different ML estimates corresponding  
%     to truncated importance functions
% Coverage = 9 x 1 vector with empirical coverages (should match the  
%     theoretical truncations of the importance function)
% NOTE: no dimension checks imposed , 16/06/04
% -----------------------------------------------------
% written by:
% Gianni Amisano, Dept of Economics
% University of Brescia
% amisano@eco.unibs.it
% 2011-08-02 modified by MJ: added subfunction logdet 
%            and changed log(det(V)) to logdet(V)
% 2015-07-23 replaced chi2inv with the subfunction chi2invx to avoid
%            dependence on the Statistics Toolbox
[M,m]=size(etaMat);
MLvec=zeros(9,1);
truncVec=0.1:0.1:0.9;
coverage=zeros(9,1);
qVec=chi2invx(truncVec,m);
etab=mean(etaMat)';
etaMat1=etaMat-repmat(etab',M,1);
V=etaMat1'*etaMat1/M;
C=inv(chol(V)');
etaMat1=etaMat1*C';
zv=sum(etaMat1.^2,2);
lnF=-0.5*(m*log(2*pi)+logdet(V)+zv); % modified by MJ: changed log(det( to logdet(
lnF=repmat(lnF,1,9)-repmat(log(truncVec),M,1);
lnFPL=lnF-repmat(lnPL,1,9);
zMat=repmat(zv,1,9)<=repmat(qVec,M,1);
for iq=1:9
    zv=zMat(:,iq);
    lnFPLv=lnFPL(:,iq);
    zv=lnFPLv(zv>0);
    coverage(iq)=length(zv)/M;
    zmean=mean(zv);
    MLvec(iq)=log(sum(exp(zv-zmean))/M)+zmean;
    MLvec(iq)=-MLvec(iq);
end
end

function y = logdet(A)
% log(det(A)) where A is positive-definite.
% This is faster and more stable than using log(det(A)).

% Written by Tom Minka
% (c) Microsoft Corporation. All rights reserved.

U = chol(A);
y = 2*sum(log(diag(U)));
end

function x = chi2invx(P,V)
% replaces the chi2inv function from the Statistics Toolbox using the
% built-in function gammaincinv
x = gammaincinv(P,V/2)*2;
end