function [draws, prior, pdata, ydata, xdata, vdata] = dfmpc6d(data, prior, gssettings, true1)
% PURPOSE: Specify the prior and simulate the posterior
% of a dynamic factor model with a Phillips curve.
% DEPENDS: kfsim or kfsim_nan, drawbsig2, drawsig2, drawbh (with the code from Joshua Chan), timing_message, compan
%          MLMGD (by Gianni Amisano), mprint
disp(mfilename)
if nargin<4, true1 = []; end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% default gssettings
if ~isfield(gssettings,'test'), gssettings.test = 0; end
if ~isfield(gssettings,'computemarglik'), gssettings.computemarglik = 0; end
if ~isfield(gssettings,'saveshocks'), gssettings.saveshocks = 0; end
if ~isfield(gssettings,'nstepirf'), gssettings.nstepirf = 0; end
if ~isfield(gssettings,'waitbar'), gssettings.waitbar = 1; end
if ~isfield(gssettings,'std0'), gssettings.std0 = 0; end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% determine the trend specification
if isfield(prior.trend,'rw') && prior.trend.rw, disp('trend: RW'); prior.trend.irw = false; prior.trend.llt = false; end
if isfield(prior.trend,'irw') && prior.trend.irw, disp('trend: IRW'); prior.trend.irw = true; prior.trend.rw = false; prior.trend.llt = false; end
if isfield(prior.trend,'llt') && prior.trend.llt, disp('trend: LLT'); prior.trend.llt = true; prior.trend.rw = false; prior.trend.irw = false; end
% by default z is a regressor not a trend of p
if ~isfield(prior.xspec,'z_is_trend'), prior.xspec.z_is_trend = 0; end
% by default no stochastic volatility
if ~isfield(prior,'sv'), prior.sv = 0; end; if prior.sv, disp('stochastic volatility'); end

% measure numbers of variables
np = size(prior.pspec,1);
ny = size(prior.yspec,1);
nx = double(isfield(prior,'xspec') && isfield(prior.xspec,'name') && ~isempty(prior.xspec.name));
nv = double(isfield(prior,'vspec') && isfield(prior.vspec,'name') && ~isempty(prior.vspec.name));

pdata = data(:,1:np); if size(pdata,2)>1, error('only one p allowed!'), end
ydata = data(:,np+1:np+ny);
xdata = data(:,np+ny+1:np+ny+nx);
vdata = data(:,np+ny+nx+1:np+ny+nx+nv);

Ttr = prior.Ttr; % to save on typing

% process training sample data
dptr = diff(pdata(1:Ttr,:));
dytr = diff(ydata(1:Ttr,:));
mean_dptr = mean(dptr(~isnan(dptr)));
mean_dytr = mean(dytr);
std_dptr = std(dptr(~isnan(dptr)));
std_dytr = std(dytr);
var_dptr = var(dptr(~isnan(dptr)));
var_dytr = var(dytr);
% x - allow missing data
mean_dxtr = nan(1,nx); var_dxtr = nan(1,nx);
for n = 1:nx
    temp = diff(xdata(~isnan(xdata(1:Ttr,n)),n)); % allow missing obs in x
    mean_dxtr(n) = mean(temp);
    var_dxtr(n) = var(temp);
end
mean_dvtr = nan(1,nv); var_dvtr = nan(1,nv);
for n = 1:nv
    temp = diff(vdata(~isnan(vdata(1:Ttr,n)),n)); % allow missing obs in v
    mean_dvtr(n) = mean(temp);
    var_dvtr(n) = var(temp);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% specify the prior

v_obs = 5;
v_state = 5;

% factor model for y
prior.sig2y.v = repmat(v_obs, 1, ny);
prior.sig2y.s = 0.25*var_dytr.*(prior.sig2y.v - 2); prior.sig2y.s(1) = 0; % set the first variance to 0

prior.igt = 2; % position of g(t) among g's
if isfield(prior,'nginy')
    ng = prior.nginy;
    prior.b.tightness = ones(1,ng);
elseif isfield(prior.b,'tightness')
    ng = length(prior.b.tightness);
else
    error('please specify either prior.nginy or prior.b.tightness')
end
prior.b.mean = zeros(ng,ny); prior.b.mean(prior.igt,1) = 1;
prior.b.var = nan(ng,ng,ny); prior.b.var(:,:,1) = 0;
for n = 2:ny
    prior.b.var(:,:,n) = var_dytr(n)/var_dytr(1).*diag(prior.b.tightness);
end

if np
    prior.ap.mean = prior.pspec.ap_mean(:);
    prior.ap.var = diag(prior.pspec.ap_tightness.^2);
    npinPC = length(prior.ap.mean);
    nr = max(npinPC,1); % number of p states
else
    npinPC = 0; nr = 0;
end

% process xspec:
% LM for z, PC coefficients of z
if isfield(prior,'xspec') && isfield(prior.xspec,'LM_b_mean') % if state z exists
    % law of motion for z
    prior.fz.mean = prior.xspec.LM_b_mean(:);
    prior.fz.var = diag(prior.xspec.LM_b_std.^2);
    nzLM = length(prior.fz.mean)-1; % subtract 1 because the intercept
    % determine how z enters the PC
    if prior.xspec.z_is_trend
        disp('z = trend inflation')
        prior.az.mean = []; prior.az.var = []; nzinPC = 0;
        nz = max(nzLM,npinPC);
    else
        disp('z is a regressor')
        prior.az.mean = prior.xspec.az_mean(:);
        if nx
            varvec = var_dptr/var_dxtr*prior.xspec.az_tightness(:).^2;
        else
            varvec = prior.xspec.az_tightness(:).^2;
        end
        prior.az.var = diag(varvec);
        nzinPC = length(prior.az.mean);
        nz = max(nzLM,nzinPC);
    end
else % state z does not exist
    prior.fz.mean = []; prior.fz.var = [];
    prior.az.mean = []; prior.az.var = [];
    nz = 0; nzinPC = 0;
end
% observation equation for x
if nx
    prior.sig2x.v = v_obs;
    prior.sig2x.s = prior.xspec.ME_sig2 * var_dxtr * (prior.sig2x.v - 2);
    prior.c.mean = prior.xspec.ME_b_mean(:);
    prior.c.var = diag(prior.xspec.ME_b_std.^2);
end

% process vspec:
% LM for q, PC coefficients of q
if nv
    % law of motion of q
    prior.fq.mean = prior.vspec.LM_b_mean(:);
    prior.fq.var = diag(prior.vspec.LM_b_std.^2);
    nqLM = length(prior.fq.mean)-1; % subtract 1 because the intercept
    if any(isnan(prior.fq.mean)) % prior based on AR estimated by OLS in the training sample
        n = 2; P = nqLM;
        x = vdata(~isnan(vdata(1:Ttr,n)),n);
        y = x(P+1:end);
        X = zeros(length(x)-P, P);
        for p = 1:P
            X(:,p) = x(P+1-p:end-p);
        end
        X = [ones(size(y)) X];
        b = X\y; bVar = inv(X'*X)*var(y - X*b);
        prior.fq.mean = b;
        prior.fq.var = 2*bVar;
    end
    % PC coefficients of q
    prior.aq.mean = prior.vspec.aq_mean(:);
    varvec = var_dptr/var_dvtr*prior.vspec.aq_tightness(:).^2;
    prior.aq.var = diag(varvec);
    nqinPC = length(prior.aq.mean);
    nq = max(nqinPC,nqLM);
else % state q does not exist
    prior.fq.mean = []; prior.fq.var = [];
    prior.aq.mean = []; prior.aq.var = [];
    nq = 0; nqinPC = 0;
end


if isfield(prior.pspec,'ag_mean')
    prior.ag.mean = prior.pspec.ag_mean(:);
    prior.ag.var = diag(var_dptr/var_dytr(1)*prior.pspec.ag_tightness.^2);
    nginPC = length(prior.ag.mean);
else
    nginPC = 0;
end

% state equations
ngLM = length(prior.phi.mean);
[g_chi, g_autocorr] = AR_properties(prior.phi.mean, ng);

% check if we keep track of sufficiently many lags of g
if ngLM>ng || nginPC>ng, error('prior.nginy (or length(prior.b.tightness)) should be >= ngLM and nginPC'); end

% trends
nw = ny;
G = eye(nw);
if prior.trend.rw
    prior.dw.mean = mean_dytr(:);
    prior.dw.var = diag(var_dytr);
    for n = 1:ny
        switch cell2mat(prior.yspec(n,3))
            case 0 % trend RW with no drift
                prior.dw.mean(n) = 0;
                prior.dw.var(n,n) = 1e-6*prior.dw.var(n,n);
            case -1 % trend i.i.d
                prior.dw.mean(n) = mean(ydata(1:Ttr,n));
                prior.dw.var(n,n) = var(ydata(1:Ttr,n));
                G(n,n) = 0;
        end
    end
    nd = 0;
else
    prior.dw.mean = []; prior.dw.var = [];
    nd = nw;
end

% Sigma - variances of the shocks to state equations
prior.tauy = cell2mat(prior.yspec(:,2)); % prior shares of variance of y explained by trend
Sigma_diag_g = (1-prior.tauy(1))/(g_chi)*var_dytr(1);
if prior.trend.irw
    Sigma_diag_w = [];
else
    Sigma_diag_w = prior.tauy(:).*var_dytr(:);
end
if prior.trend.llt || prior.trend.irw
    if isfield(prior.trend,'Hdname') && ~isempty(prior.trend.Hdname)
        switch prior.trend.Hdname, case 'aa', Hd = 5e1; case 'a', Hd = 1e2; case 'b', Hd = 5e2; case 'c', Hd = 1e3'; case 'd', Hd = 1e4; case 'e', Hd = 1e5; otherwise error(['prior.trend.Hdname not recognized, ' prior.trend.Hdname]), end
        prior.trend.Hdvec = repmat(Hd,nd,1);
    else
        prior.trend.Hdname = 'mix';
        prior.trend.Hdvec = cell2mat(prior.yspec(:,4));
        Hd = prior.trend.Hdvec(:)';
    end
    Sigma_diag_d = prior.trend.Hdvec.^(-1);
    disp(['prior Hd name: ' prior.trend.Hdname ', value: ' num2str(Hd) ])
else
    Sigma_diag_d = []; prior.trend.Hdname = '';
end
if nz
    if nx % if there is an observable (x), sig2 scales the variance of x
        if isfield(prior.xspec,'LM_sig2'), frac = prior.xspec.LM_sig2; else frac = 0.5; end
        Sigma_diag_z = frac*var_dxtr(:);
    else % otherwise sig2 is the variance
        Sigma_diag_z = prior.xspec.LM_sig2;
    end
    Sigma_diag_z_name = {'z'};
else
    Sigma_diag_z = []; Sigma_diag_z_name = {};
end
if nq
    if isfield(prior.vspec,'LM_sig2'), frac = prior.vspec.LM_sig2; else frac = 0.5; end
    Sigma_diag_q = frac*var_dvtr(:);
    Sigma_diag_q_name = cellstr(num2str((1:length(Sigma_diag_q))', 'q%d'));
else
    Sigma_diag_q = []; Sigma_diag_q_name = {};
end
if np
    Sigma_diag_p = 0.25*var_dptr;
else
    Sigma_diag_p = [];
end

ngs = length(Sigma_diag_g);
nws = length(Sigma_diag_w);
nds = length(Sigma_diag_d);
nzs = length(Sigma_diag_z);
nqs = length(Sigma_diag_q);
nps = length(Sigma_diag_p);


prior.Sigma.diag = [Sigma_diag_g Sigma_diag_w' Sigma_diag_d' Sigma_diag_z' Sigma_diag_q' Sigma_diag_p];
prior.Sigma.names = [{'g'}; strseq('w',1:nws); strseq('d',1:nds); Sigma_diag_z_name; Sigma_diag_q_name; {'p'}];

prior.Sigma.v = v_state;
prior.Sigma.S = diag(prior.Sigma.diag*(prior.Sigma.v - 2));

% initial state
a1_g = zeros(ng,1);
std_g = 5*std_dytr(1);
C1_g = chol(std_g^2*toeplitz(g_autocorr(1:ng)))';

a1_w = ydata(Ttr,:)';
C1_w = 5*diag(std_dytr); C1_w(1,1) = std_g;

if prior.trend.llt || prior.trend.irw
    a1_d = mean_dytr(1:ny)';
    C1_d = diag(5*std_dytr(1:ny));
else
    a1_d = []; C1_d = [];
end

a1_z = []; C1_z = [];
a1_q = []; C1_q = [];
a1_p = []; C1_p = [];
if np
    % prior for (z(1),z(0),...)
    if nz
        if nx
            a1_z = flipud(xdata(Ttr+1-nz+1:Ttr+1,1));
            C1_z = 1e-6*eye(nz);
            % replace z(1) with the RW forecast based on x(0)
            % (otherwise we would be using the first observation of the sample in the prior)
            a1_z(1) = xdata(Ttr);
            C1_z(1,1) = sqrt(var_dxtr);
        else
            a1_z = flipud(pdata(Ttr+1-nz+1:Ttr+1,1));
            C1_z = 1e-6*eye(nz);
            % replace z(1) with the RW forecast based on p(0)
            % (otherwise we would be using the first observation of the sample in the prior)
            a1_z(1) = pdata(Ttr);
            C1_z(1,1) = sqrt(var_dptr);
        end
    end
    
    % prior for (q(1),q(0),...)
    if nq
        a1_q = flipud(vdata(Ttr+1-nq+1:Ttr+1,1));
        C1_q = 1e-6*eye(nq);
        % replace q(1) with the RW forecast based on v(0)
        % (otherwise we would be using the first observation of the sample in the prior)
        a1_q(1) = vdata(Ttr);
        C1_q(1,1) = sqrt(var_dvtr);
    end
    
    % prior for (p(1),p(0),...)
    a1_p = flipud(pdata(Ttr+1-nr+1:Ttr+1)); % assumes nr periods of p in the state vector
    C1_p = 1e-6*eye(nr);
    % replace p(1) with the RW forecast based on p(0)
    % (otherwise we would be using the first observation of the sample in the prior)
    a1_p(1) = pdata(Ttr);
    C1_p(1,1) = std_dptr;
end

a1 = [a1_g; a1_w; a1_d; a1_z; a1_q; a1_p];
C1 = blkdiag(C1_g, C1_w, C1_d, C1_z, C1_q, C1_p);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% estimation

% starting value of the parameters for the Gibbs sampler - from the prior
ag = zeros(0,nginPC); az = zeros(0,nzinPC); aq = zeros(0,nqinPC); ap = zeros(0,npinPC);
if np
    if nginPC, ag = prior.ag.mean + gssettings.std0*chol(prior.ag.var)'*randn(nginPC,1); end
    if nzinPC, az = prior.az.mean + gssettings.std0*chol(prior.az.var)'*randn(nzinPC,1); end
    if nqinPC, aq = prior.aq.mean + gssettings.std0*chol(prior.aq.var)'*randn(nqinPC,1); end
    if np, ap = prior.ap.mean + gssettings.std0*chol(prior.ap.var)'*randn(npinPC,1); end
end

sig2y = prior.sig2y.s ./ (prior.sig2y.v - 2);

B = prior.b.mean;
for n = 2:ny
    B(:,n) = B(:,n) + gssettings.std0*chol(prior.b.var(:,:,n))'*randn(ng,1);
end

if nx
    c = prior.c.mean + gssettings.std0*chol(prior.c.var)'*randn(length(prior.c.mean),1);
    sig2x = prior.sig2x.s ./ (prior.sig2x.v - 2);
else
    sig2x = []; c = [];
end

phi = prior.phi.mean + gssettings.std0*chol(prior.phi.var)'*randn(ngLM,1);
if prior.trend.rw
    dw = prior.dw.mean + gssettings.std0*chol(prior.dw.var)'*randn(nw,1);
else
    dw = zeros(nw,1);
end
fz = []; if nz, fz = prior.fz.mean + gssettings.std0*chol(prior.fz.var)'*randn(nzLM+1,1); end
fq = []; if nq, fq = prior.fq.mean + gssettings.std0*chol(prior.fq.var)'*randn(nqLM+1,1); end

Sigma = prior.Sigma.S/(prior.Sigma.v - 2);

% default settings regarding the sample to use and forecasts
if ~isfield(prior,'S'), prior.S = size(pdata,1); end
if ~isfield(prior,'H'), prior.H = 0; end
if ~isfield(prior,'R'), prior.R = 0; end

% estimation data
pest = pdata(Ttr+1:prior.S,:);
yest = ydata(Ttr+1:prior.S,:);
xest = xdata(Ttr+1:prior.S,:);
vest = vdata(Ttr+1:prior.S,:);
est = [pest yest xest vest]; % est holds the observables, possibly with NaNs
Test = size(est,1);
disp(['est.data(1,1) = ' num2str(est(1,1))])
disp(['est.data(S,1) = ' num2str(est(end,1))])

% choose the Kalman filter function
kffun = @kfsim_nan_sv;
tnan = find(isnan(sum(est,2)));

% prepare trueq: the quantity we forecast
A = tril(repmat(1./(1:prior.H+1)',1,prior.H+1)); % matrix that converts pp into q
if size(true1,2)
    y1 = true1(prior.S:end,:);
    y1 = [y1; nan(prior.H + 1 - size(y1,1), size(y1,2))]; % add NaNs if needed
    y1 = y1(1:1+prior.H,:); % truncate if needed
    y1_unavailable = isnan(y1);
    y1(y1_unavailable) = 0; % convert to 0 so we can use matrix A
    trueq = A*y1;
    trueq(y1_unavailable) = NaN;
else
    trueq = zeros(prior.H + 1,0);
end


% prepare for the Gibbs sampler
% simulation length - derived from burnin, ndraws, saveevery
ndraws = gssettings.ndraws;
nalldraws = gssettings.burnin + ndraws * gssettings.saveevery;

draws.ag = nan(ndraws,length(ag));
draws.az = nan(ndraws,length(az));
draws.aq = nan(ndraws,length(aq));
draws.ap = nan(ndraws,length(ap));
draws.b = nan(ndraws,ng*ny);
draws.sig2y = nan(ndraws,ny);
draws.c = nan(ndraws,length(c));
draws.sig2x = nan(ndraws,nx);
draws.phi = nan(ndraws,length(phi));
draws.d = nan(ndraws,nw-nd); % rw: nw-nd=nw llt: nw-nd=0
draws.fz = nan(ndraws,length(fz));
draws.fq = nan(ndraws,length(fq));
nSig = size(prior.Sigma.S,1);
draws.Sigma = nan(ndraws,nSig);
draws.states = nan(Test+prior.H, length(a1), ndraws);
draws.logpl = nan(ndraws,1);
draws.irf = nan(gssettings.nstepirf, np+ny+nx+nv+length(a1), ndraws, 1+nx+nv);
draws.obs = nan(prior.R+1+prior.H, np+ny+nx+nv, ndraws);
draws.q_mean = nan(prior.H+1, ndraws);
draws.q_var = nan(prior.H+1, ndraws);
draws.q_dens = nan(prior.H+1, size(true1,2), ndraws);
if gssettings.saveshocks
    draws.eps = nan(Test, np+ny+nx, ndraws);
    draws.eta = nan(Test, length(a1), ndraws);
end

if prior.sv
    % priors
    if ~isfield(prior,'h'), prior.h.Vh = 10; prior.h.b0 = 0; prior.h.Vh0 = 10; prior.h.Vomegah = .2; end
    if ~isfield(prior,'g'), prior.g.Vh = 10; prior.g.b0 = 0; prior.g.Vh0 = 10; prior.g.Vomegah = .2; end
    
    % initialize the Markov chain
    h0 = log(var_dptr)/2;
    g0 = log(var_dptr)/2;
    omegah = sqrt(.2);
    omegag = sqrt(.2);
    htilde = zeros(Test,1);
    gtilde = zeros(Test,1);
    hh = h0 + omegah*htilde;
    gg = g0 + omegag*gtilde;
    
    % define a few things
    npts = 500; % number of grid points
    draws.omegah.grid = linspace(-1,1,npts)';
    draws.omegag.grid = linspace(-1,1,npts)';
    
    % initialize for storage
    draws.theta = nan(ndraws,4); % [omegah2 omegag2 h0 g0]
    draws.h = zeros(ndraws,Test+prior.H);
    draws.g = zeros(ndraws,Test+prior.H);
    draws.omegah.dens = zeros(npts,1);
    draws.omegag.dens = zeros(npts,1);
    draws.lpden = zeros(ndraws,3); % log posterior densities of omegah = valh; omegag = valg; omegah = omegag
end


% Gibbs sampler
it_save = 0;
if gssettings.waitbar, waitbar_handle = waitbar(0,'Gibbs sampler','Name','Gibbs sampler running'); end
timing_start = now;
disp(['start: ',datestr(timing_start,0), '; total iterations: ', num2str(nalldraws)])
for it = 1:nalldraws
    logprior = 0;
    
    % update the state-space model matrices
    
    % observation equation:
    
    if np
        ZZp = zeros(1,length(a1)); ZZp(end-nr+1) = 1;
        ZZp(1,ng+nw+nd+1) = prior.xspec.z_is_trend;
    else
        ZZp = [];
    end
    
    ZZy = [B', eye(nw), zeros(ny,nd+nz+nq+nr)]; % eqn for y
    
    if nx
        ddx = c(1);
        ZZx = [ zeros(nx, ng+nw+nd), c(2), zeros(nx,nz-1+nq+nr)];
    else
        ddx = [];
        ZZx = [];
    end
    
    if nv
        ZZv = [ zeros(nv, ng+nw+nd+nz), 1, zeros(nv,nq-1+nr)];
    else
        ZZv = [];
    end
    ZZ = [ZZp; ZZy; ZZx; ZZv];
    
    dd = [zeros(np+ny,1); ddx; zeros(nv,1)];
    
    WW = diag(sqrt([zeros(1,np) sig2y sig2x zeros(1,nv)]));
    WW(:,~logical(sum(WW))) = []; % drop shocks that have zero variance
    
    % state equation:
    % use: dw, phi, f1, fq, Sigma, ag, az, aq, ap
    [Fg, dg] = compan([0; phi], ng);
    [Fz, dz] = compan(fz, nz);
    [Fq, dq] = compan(fq, nq);
    [Fp, dp] = compan([zeros(np,1); ap], nr);
    
    cc = [dg; dw; zeros(nd,1); dz; dq; dp];
    
    TT = [Fg zeros(ng, nw + nd + nz + nq + nr);
        zeros(nw,ng) G eye(nd) zeros(nw,nz + nq + nr);
        zeros(nd, ng + nw) eye(nd) zeros(nd,nz + nq + nr);
        zeros(nz, ng + nw + nd) Fz zeros(nz, nq + nr);
        zeros(nq, ng + nw + nd + nz) Fq zeros(nq, nr);
        ag' zeros(np,ng-nginPC + nw + nd) az' zeros(np,nz-nzinPC) aq' zeros(np,nq-nqinPC) Fp(1:np,:);
        zeros(nr-1, ng + nw + nd + nz + nq) Fp(2:end,:)];
    
    shocks_in_g = ones(ng,1); shocks_in_g(2:end) = 0;
    shocks_in_w = zeros(nw,1); shocks_in_w(1:nws) = 1; % zeros or ones
    shocks_in_z = ones(nz,1); shocks_in_z(2:end) = 0;
    shocks_in_q = ones(nq,1); shocks_in_q(2:end) = 0;
    shocks_in_p = ones(nr,1); shocks_in_p(2:end) = 0;
    shocks_in_state_eq = logical([shocks_in_g; shocks_in_w; ones(nd,1); shocks_in_z; shocks_in_q; shocks_in_p]);
    RRR = zeros(length(a1),nSig,Test); RRR(shocks_in_state_eq,:,:) = repmat(chol(Sigma)',1,1,Test);
    if prior.sv
        i_shock_z = sum([shocks_in_g; shocks_in_w; ones(nd,1)])+1; % position of the z shock in Sigma
        i_shock_p = sum([shocks_in_g; shocks_in_w; ones(nd,1); shocks_in_z; shocks_in_q;])+1; % position of the p shock in Sigma
        RRR(ng+nw+nd+1,i_shock_z,:) = permute(exp(0.5*gg),[2 3 1]);
        RRR(ng+nw+nd+nz+nq+1,i_shock_p,:) = permute(exp(0.5*hh),[2 3 1]);
    end
    
    % in test mode just plot the filtered states and the likelihood at the
    % starting values of parameters and return
    if gssettings.test
        [aaa, loglik] = kffun(est', dd, ZZ, WW, cc, TT, RRR, a1, C1, 1);
        aaa = aaa';
        disp(sum(loglik))
        tt = (1:size(ydata,1))';
        if np % plot inflation
            figure()
            hold on
            plot(tt, pdata, '-k') % plot p
            plot(tt(Ttr+1:prior.S), aaa(:,ng+ny+1), '-b') % plot z
            hold off
            title(prior.pspec.name, 'Interpreter', 'none')
        end
        % plot first y
        figure()
        hold on
        plot(tt, ydata(:,1), '-k')
        plot(tt(Ttr+1:prior.S), aaa(:,ng+1), '-b')
        hold off
        title(prior.yspec{1,1}, 'Interpreter', 'none')
        % plot output gap
        figure()
        hold on
        plot(tt(Ttr+1:prior.S), zeros(size(aaa,1),1), '-k')
        AX = plotyy(tt(Ttr+1:prior.S), aaa(:,1), tt(Ttr+1:prior.S), loglik');
        hold off
        title('output gap', 'Interpreter', 'none')
        if gssettings.waitbar, close(waitbar_handle); end
        return
    end
    
    % draw states conditionally on the parameters
    aaa = kffun(est', dd, ZZ, WW, cc, TT, RRR, a1, C1, 2)';
    
    % append a draw of states in S+1 - needed for the regressions with lags
    aaa1 = [aaa; (cc + TT*aaa(end,:)' + RRR(:,:,end)*randn(size(RRR,2),1))'];
    
    % draw observables conditionally on the states and parameters
    obsdraw = est;
    for i = 1:length(tnan)
        t = tnan(i);
        inan = find(isnan(est(t,:)));
        obsdraw(t,inan) = (dd(inan) + ZZ(inan,:)*aaa(t,:)' + WW(inan,:)*randn(size(WW,2),1))';
    end
    if np, pest = obsdraw(:,np); end
    yest = obsdraw(:,np+(1:ny));
    xest = obsdraw(:,np+ny+(1:nx));
    vest = obsdraw(:,np+ny+nx+(1:nv));
    
    if gssettings.saveshocks % compute shocks
        epsdraw = [pest yest xest vest] - repmat(dd',Test,1) - aaa*ZZ';
        etadraw = aaa1(2:end,:) - repmat(cc',Test,1) - aaa*TT';
    end
    
    
    % draw parameters conditionally on the states and observables
    
    % parameters of the factor model for y
    for n = 2:ny % n indexes variables in y
        y = yest(:,n) - aaa(:,ng+n); % select y(t) - w(t) for the nth variable in y
        X = aaa(:,1:ng); % select lags of g
        
        % B - nth column and sig2y(n)
        [B(:,n), sig2y(n), logpriorbsig2] = drawbsig2(y, X, sig2y(n), ...
            prior.b.mean(:,n), prior.b.var(:,:,n), prior.sig2y.s(n), prior.sig2y.v(n));
        logprior = logprior + logpriorbsig2;
    end
    
    % parameters of the model for x
    for n = 1:nx
        y = xest(:,n);
        X = [ones(Test,1) aaa(:,ng+nw+nd+1)];
        [c, sig2x, logpriorbsig2] = drawbsig2(y, X, sig2x, ...
            prior.c.mean, prior.c.var, prior.sig2x.s, prior.sig2x.v);
        logprior = logprior + logpriorbsig2;
    end
    
    % parameters of the state equations
    
    priorv = prior.Sigma.v; % this is common to all state equations
    
    % gap equation - AR(ngLM)
    y = aaa1(2:end,1); X = aaa(:,1:ngLM);
    sig2 = Sigma(1,1); priors = prior.Sigma.S(1,1);
    priorvar = prior.phi.var; priorm = prior.phi.mean;
    [phi, Sigma(1,1), logpriorbsig2] = drawbsig2(y, X, sig2, priorm, priorvar, priors, priorv);
    logprior = logprior + logpriorbsig2;
    
    % w equations
    if prior.trend.rw % estimate the constant dw and the shock variance
        for n = 1:nws
            % dw(n)
            if G(n,n)
                y = diff(aaa1(:,ng+n)); X = ones(Test,1);
            else
                y = aaa1(2:end,ng+n); X = ones(size(y));
            end
            i_shock = ngs + n;
            sig2 = Sigma(i_shock,i_shock); priors = prior.Sigma.S(i_shock,i_shock);
            priorvar = prior.dw.var(n,n); priorm = prior.dw.mean(n);
            [dw(n), Sigma(i_shock,i_shock), logpriorbsig2] = drawbsig2(y, X, sig2, priorm, priorvar, priors, priorv);
            logprior = logprior + logpriorbsig2;
        end
    else % estimate only the shock variance
        for n = 1:nws
            % dw(n)
            if G(n,n)
                u = diff(aaa1(:,ng+n)) - aaa(:,ng+nw+n);
            else
                u = aaa1(2:end,ng+n) - aaa(:,ng+nw+n); disp('********');
            end
            i_shock = ngs + n;
            priors = prior.Sigma.S(i_shock,i_shock);
            [Sigma(i_shock,i_shock), logpriorsig2] = drawsig2(u, priors, priorv);
            logprior = logprior + logpriorsig2;
        end
    end
    
    % d equations - only the shock variance
    if ~isfield(prior.trend,'fixsig2d') || ~prior.trend.fixsig2d
        for n = 1:nds
            u = diff(aaa1(:,ng+nw+n));
            i_shock = ngs + nws + n;
            priors = prior.Sigma.S(i_shock,i_shock);
            [Sigma(i_shock,i_shock), logpriorsig2] = drawsig2(u, priors, priorv);
            logprior = logprior + logpriorsig2;
        end
    end
    
    % z equation
    if nz
        y = aaa1(2:end,ng+nw+nd+1); X = [ones(Test,1) aaa(:,ng+nw+nd+(1:nzLM))];
        priorvar = prior.fz.var; priorm = prior.fz.mean;
        if prior.sv
            [fz, gtilde, g0, omegag, omegaghat, Domegag, logpriorbsig2] = drawbh(y, X, gtilde, g0, omegag, priorm, priorvar, prior.g);
            gg = g0 + omegag*gtilde;
        else
            i_shock = ngs + nws + nds + 1;
            sig2 = Sigma(i_shock,i_shock); priors = prior.Sigma.S(i_shock,i_shock);
            [fz, Sigma(i_shock,i_shock), logpriorbsig2] = drawbsig2(y, X, sig2, priorm, priorvar, priors, priorv);
        end
        logprior = logprior + logpriorbsig2;
    end
    % q equation
    if nq
        y = aaa1(2:end,ng+nw+nd+nz+1); X = [ones(Test,1) aaa(:,ng+nw+nd+nz+(1:nqLM))];
        i_shock = ngs + nws + nds + nzs + 1;
        sig2 = Sigma(i_shock,i_shock); priors = prior.Sigma.S(i_shock,i_shock);
        priorvar = prior.fq.var; priorm = prior.fq.mean;
        [fq, Sigma(i_shock,i_shock), logpriorbsig2] = drawbsig2(y, X, sig2, priorm, priorvar, priors, priorv);
        logprior = logprior + logpriorbsig2;
    end
    % p equation
    if np
        y = aaa1(2:end,ng+nw+nd+nz+nq+1);
        X = aaa(:, [1:nginPC ng+nw+nd+(1:nzinPC) ng+nw+nd+nz+(1:nqinPC) ng+nw+nd+nz+nq+(1:npinPC)]);
        priorm = [prior.ag.mean; prior.az.mean; prior.aq.mean; prior.ap.mean]; priorvar = blkdiag(prior.ag.var, prior.az.var, prior.aq.var, prior.ap.var);
        if prior.sv
            [a, htilde, h0, omegah, omegahhat, Domegah, logpriorbsig2] = drawbh(y, X, htilde, h0, omegah, priorm, priorvar, prior.h);
            hh = h0 + omegah*htilde;
        else
            i_shock = ngs + nws + nds + nzs + nqs + 1;
            sig2 = Sigma(i_shock,i_shock); priors = prior.Sigma.S(i_shock,i_shock);
            [a, Sigma(i_shock,i_shock), logpriorbsig2] = drawbsig2(y, X, sig2, priorm, priorvar, priors, priorv);
        end
        logprior = logprior + logpriorbsig2;
        ag = a(1:nginPC);
        az = a(nginPC+(1:nzinPC));
        aq = a(nginPC+nzinPC+(1:nqinPC));
        ap = a(nginPC+nzinPC+nqinPC+(1:npinPC));
    end
    
    % report progress
    if gssettings.waitbar && ~rem(it,gssettings.saveevery)
        waitbar(it/nalldraws, waitbar_handle, timing_message(it, nalldraws, timing_start))
    end
    % save current iteration if appropriate
    if it>gssettings.burnin && ~rem(it,gssettings.saveevery)
        it_save = it_save + 1;
        if gssettings.computemarglik
            [temp, loglik] = kffun(est', dd, ZZ, WW, cc, TT, RRR, a1, C1, 0);
            draws.logpl(it_save) = sum(loglik,2) + logprior;
        end
        if np
            draws.ag(it_save,:) = ag';
            draws.az(it_save,:) = az';
            draws.aq(it_save,:) = aq';
            draws.ap(it_save,:) = ap';
        end
        draws.b(it_save,:) = B(:)';
        draws.sig2y(it_save,:) = sig2y';
        if nx
            draws.c(it_save,:) = c';
            draws.sig2x(it_save,:) = sig2x';
        end
        if nz
            draws.fz(it_save,:) = fz';
        end
        if nq
            draws.fq(it_save,:) = fq';
        end
        draws.phi(it_save,:) = phi';
        draws.d(it_save,:) = dw(1:nw-nd,1)';
        draws.Sigma(it_save,:) = diag(Sigma)';
        draws.states(1:Test,:,it_save) = aaa;
        
        if prior.sv
            draws.g(it_save,1:Test) = gg';
            draws.h(it_save,1:Test) = hh';
            draws.theta(it_save,:) = [omegah^2 omegag^2 h0 g0];
            lh0 = -.5*log(2*pi*Domegah) - .5*(omegahhat-0)^2/Domegah;
            lg0 = -.5*log(2*pi*Domegag) - .5*(omegaghat-0)^2/Domegag;
            lhg0 = lh0 + lg0;
            draws.lpden(it_save,:) = [lh0 lg0 lhg0];
            draws.omegah.dens = draws.omegah.dens + normpdf(draws.omegah.grid,omegahhat,sqrt(Domegah));
            draws.omegag.dens = draws.omegag.dens + normpdf(draws.omegag.grid,omegaghat,sqrt(Domegag));
        end
        
        if prior.H % compute a forecast
            % compute a draw of the observables y and states a in S,...,S+H
            RRRSH = repmat(RRR(:,:,end),1,1,prior.H+1);
            if prior.sv
                gtilSH = gtilde(end) + [0; cumsum(randn(prior.H,1))];
                htilSH = htilde(end) + [0; cumsum(randn(prior.H,1))];
                ggSH = g0 + omegag*gtilSH;
                hhSH = h0 + omegah*htilSH;
                draws.g(it_save,Test:end) = ggSH';
                draws.h(it_save,Test:end) = hhSH';
                RRRSH(ng+nw+nd+1,i_shock_z,:) = permute(exp(0.5*ggSH),[2 3 1]);
                RRRSH(ng+nw+nd+nz+nq+1,i_shock_p,:) = permute(exp(0.5*hhSH),[2 3 1]);
            end
            ySH = [obsdraw(end,:)', nan(np+ny+nx+nv, prior.H)];
            aSH = [aaa(end,:)', nan(size(aaa,2), prior.H)];
            for h = 1:prior.H
                aSH(:,h+1) = cc + TT*aSH(:,h) + RRRSH(:,:,h)*randn(size(RRR,2),1);
                ySH(:,h+1) = dd + ZZ*aSH(:,h+1) + WW*randn(size(WW,2),1);
            end
            % append y to observables in S-R,...,S-1 and a to states in 1,...S
            draws.obs(:,:,it_save) = [obsdraw(end-prior.R:end-1,:);  ySH'];
            draws.states(Test+1:Test+prior.H,:,it_save) = aSH(:,2:end)';
            
            % Now compute the mean and variance of q,
            % defined as the average inflation in periods S,S+1,...,S+H
            nstates = length(cc);
            % distribution of future states: ctil, Tautil, Rtil (see derivations in preddens.pdf)
            if ~isnan(est(end,1)) % here the density is conditional on states up to S
                ctil = [aaa(end,:)'; repmat(cc,prior.H,1)];
                Tautil = blktridiag(eye(nstates), -TT, zeros(nstates,nstates), prior.H+1);
                Rtil = blktridiag(RRRSH, zeros(size(RRRSH(:,:,1:end-1))), zeros(size(RRRSH(:,:,1:end-1))));
                Rtil(1:size(RRR,1),1:size(RRR,2)) = 0;
            else % here the density is conditional on states up to S-1
                nstates = length(cc);
                iobs = find(~isnan(est(end,:))); ddi = dd(iobs,:); ZZi = ZZ(iobs,:); WWi = WW(iobs,:); yyi = est(end,iobs)';
                RR = RRR(:,:,end-1);
                % E(a(S)|y(S),a(S-1)
                temp = cc + TT*aaa(end-1,:)' + RR*RR'*ZZi'/(ZZi*RR*RR'*ZZi'+WWi*WWi')*(yyi - (ddi + ZZi*cc + ZZi*TT*aaa(end-1,:)'));
                ctil = [temp; repmat(cc,prior.H,1)];
                Tautil = blktridiag(eye(nstates), -TT, zeros(nstates,nstates), prior.H+1);
                % Var(a(S)|y(S),a(S-1)
                temp = (RR*RR') - (RR*RR')*ZZi'/(((ZZi*RR)*(ZZi*RR)')+(WWi*WWi'))*ZZi*(RR*RR');
                i_unknown_states = logical(sum(temp));
                temp = temp(i_unknown_states,i_unknown_states); % take only the state equations with shocks
                Rtil = blktridiag(RRRSH, zeros(size(RRRSH(:,:,1:end-1))), zeros(size(RRRSH(:,:,1:end-1))));
                try Rtil(i_unknown_states,i_unknown_states(shocks_in_state_eq)) = chol(temp)';
                catch [ranktemp, temp] = factorise_semidefinitem(temp); Rtil(i_unknown_states,i_unknown_states(shocks_in_state_eq)) = temp; disp('*');
                end
            end
            mualf = Tautil\ctil;
            Valf = Tautil\Rtil; Valf = Valf*Valf';
            % distribution of future observable no. 1
            dtil = repmat(dd(1),prior.H+1,1);
            Ztil = blktridiag(ZZ(1,:), zeros(1,nstates), zeros(1,nstates), prior.H+1);
            if np, Wtil = 0; else Wtil = diag(WW(1,1)*ones(prior.H+1,1)); end
            mupi = dtil + Ztil*mualf; Vpi = Ztil*Valf*Ztil' + Wtil*Wtil';
            % distribution of q
            muq = A*mupi; Vq = A*Vpi*A'; vq = diag(Vq);
            % store
            draws.q_mean(:,it_save) = muq;
            draws.q_var(:,it_save) = vq;
            for i = 1:size(trueq,2)
                draws.q_dens(:,i,it_save) = ((6.2832*vq).^-0.5).*exp(-0.5*((trueq(:,i) - muq).^2)./vq);
            end
        end
        if gssettings.nstepirf>0 % compute irf
            e = zeros(nSig,3);
            e(1,1) = 1; % g shock
            if nz, e(ngs+nws+nds+1,2) = 1; end% z shock
            if nq, e(ngs+nws+nds+nzs+1,3) = 1; end% q shock
            e(:,~logical(sum(e))) = []; % drop z or q shock if missing
            for j = 1:size(e,2);
                e1 = e(:,j);
                aatemp = RRR(:,:,end)*e1;
                draws.irf(1,:,it_save,j) = [ZZ*aatemp; aatemp]';
                for h = 2:gssettings.nstepirf
                    aatemp = TT*aatemp;
                    draws.irf(h,:,it_save,j) = [ZZ*aatemp; aatemp]';
                end
            end
        end
        if gssettings.saveshocks
            draws.eps(:,:,it_save) = epsdraw;
            draws.eta(:,:,it_save) = etadraw;
        end
    end
end
if gssettings.waitbar, close(waitbar_handle), end
disp(timing_message(it, nalldraws, timing_start))
disp('finished Gibbs sampling')

% collect draws of parameters for marginal likelihood and convergence diagn
drawslogpl = draws.logpl;
drawsB = draws.b; drawsB(:,1:ng) = [];
drawssig2y = draws.sig2y; drawssig2y(:,1) = [];
drawssig2y = log(drawssig2y);
drawslogpl = drawslogpl + sum(drawssig2y,2); % Jacobian correction
drawssig2x = log(draws.sig2x);
drawslogpl = drawslogpl + sum(drawssig2x,2); % Jacobian correction
drawsSigma = log(draws.Sigma); iSigmafixed = ~logical(max(drawsSigma)-min(drawsSigma)); drawsSigma(:,iSigmafixed) = []; % drop any Sigmas that were fixed
drawslogpl = drawslogpl + sum(log(draws.Sigma),2); % Jacobian correction
drawsPar = [drawsB drawssig2y draws.c drawssig2x draws.phi draws.d draws.fz draws.fq drawsSigma];
drawsParnames = [strseq('b',ng+1:ng*ny); strseq('logsig2y',2:ny); strseq('c',1:size(draws.c,2)); strseq('logsig2x',1:nx); 'phi1'; 'phi2';
    strseq('d',1:size(draws.d,2)); strseq('fz',1:size(draws.fz,2)); strseq('fq',1:size(draws.fq,2)); strcat('logsig_',prior.Sigma.names(~iSigmafixed))];
if np
    drawsPar = [drawsPar draws.ag draws.az draws.aq draws.ap];
    drawsParnames = [drawsParnames; strseq('ag',1:ng); strseq('az',1:nzinPC); strseq('aq',1:nqinPC); strseq('ap',1:npinPC)];
end
if prior.sv
    drawsPar = [drawsPar log(draws.theta(:,1:2)) draws.theta(:,3:4)];
    drawsParnames = [drawsParnames; {'logomegah2';'logomegag2';'h0';'g0'}];
end
draws.drawslogpl = drawslogpl;
draws.drawsPar = drawsPar; clear drawsPar;
draws.drawsParnames = drawsParnames;

if gssettings.computemarglik
    % compute the marginal likelihood
    [lnmarglik_vec, lnmarglik_coverage] = MLMGD(draws.drawsPar, draws.drawslogpl);
    in.cnames = strvcat('marglik','coverage');
    mprint([lnmarglik_vec, lnmarglik_coverage],in);
    draws.lnmarglik = mean(lnmarglik_vec);
    disp(['marginal likelihood (Geweke method): ' num2str(draws.lnmarglik)])
end

if prior.sv
    maxlpden = max(draws.lpden);
    lpostden = log(mean(exp(draws.lpden-repmat(maxlpden,ndraws,1)))) + maxlpden;
    lpriden = [log(normpdf(0,0,sqrt(prior.h.Vomegah))) log(normpdf(0,0,sqrt(prior.g.Vomegah)))];
    lpriden(3) = sum(lpriden(1:2));
    draws.lBF = lpriden-lpostden;
    % report log BFs
    fprintf('\n');
    fprintf('log Bayes Factor\n');
    fprintf('log BF_uh       | %.1f \n', draws.lBF(1));
    fprintf('log BF_ug       | %.1f \n', draws.lBF(2));
    fprintf('log BF_u,gh     | %.1f \n', draws.lBF(3));
end

if prior.H
    draws.A = A;
    draws.q_true = trueq;
    in.cnames = strvcat(['h'; strseq('trueq',1:size(trueq,2))]);
    in.fmt = strvcat(['%12d'; repmat({'%10.4f'},size(trueq,2),1)]);
    % compute point forecasts (posterior median of the predictive density)
    draws.q_med = A*median(draws.obs(prior.R+1:end,1,:),3); % median is not a linear operator so this is wrong, but the error is small
    draws.q_err = trueq - draws.q_med*ones(1,size(trueq,2));
    disp('point forecast errors: trueq - median forecast of q')
    mprint([(0:prior.H)' draws.q_err],in)
    % compute the logscore
    draws.q_logscores = log(mean(draws.q_dens,3));
    disp('log predictive scores')
    mprint([(0:prior.H)' draws.q_logscores],in);
    
    % This is a check: compute the logscore from draws.q_mean and draws.obsvar
    % The result should be the same as draws.logscore, but this example shows how
    % to compute the logscore for some other trueq if needed.
    for i = 1:size(trueq,2)
        temp = (draws.q_mean - repmat(trueq(:,i),1,ndraws)).^2;
        temp = exp(-0.5*temp./draws.q_var);
        temp = (6.2832*draws.q_var).^(-0.5).*temp;
        temp = mean(temp,2); % average over ndraws gaussian densities
        draws.q_logscores1(:,i) = log(temp);
    end
end
end