% This script estimates an UCSV model with gamma priors on the error
% variances in the two SV state equations.
%
% See:
% Chan, J.C.C. (2016). Specification Tests for Time-Varying Parameter
% Models with Stochastic Volatility, Econometric Reviews, forthcoming

% Modified by Marek Jarocinski July 2016 for compatibility with other
% programs in the output gap project


function [draws, prior, pdata, ydata, xdata, vdata] = estimate_ucsv(data, prior, gssettings, true1)
disp(mfilename)
if nargin<4, true1 = []; end

% default settings regarding the sample to use and forecasts
if ~isfield(prior,'S'), prior.S = size(data,1); end
if ~isfield(prior,'H'), prior.H = 0; end
if ~isfield(prior,'R'), prior.R = 0; end

% estimation data
y = data(prior.Ttr+1:prior.S,:);
pdata = y; ydata = []; xdata = []; vdata = [];
Test = size(y,1); % Test - size of the nominal estimation sample, including a NaN in period S
if isnan(y(end))
    y = data(prior.Ttr+1:prior.S-1,:);
end
T = size(y,1); % T - size of the actual sammple with only nonmissing data
H = speye(T) - sparse(2:T,1:(T-1),ones(1,T-1),T,T);
tau0 = 0; Vtau = 10;

% prepare trueq: the quantity we forecast
A = tril(repmat(1./(1:prior.H+1)',1,prior.H+1)); % matrix that converts pp into q
if size(true1,2)
    y1 = true1(prior.S:end,:);
    y1 = [y1; nan(prior.H + 1 - size(y1,1), size(y1,2))]; % add NaNs if needed
    y1 = y1(1:1+prior.H,:); % truncate if needed
    y1_unavailable = isnan(y1);
    y1(y1_unavailable) = 0; % convert to 0 so we can use matrix A
    trueq = A*y1;
    trueq(y1_unavailable) = NaN;
else
    trueq = zeros(prior.H + 1,0);
end


% prepare for the Gibbs sampler
% simulation length - derived from burnin, ndraws, saveevery
ndraws = gssettings.ndraws;
nalldraws = gssettings.burnin + ndraws * gssettings.saveevery;

draws.ag = []; % this is for compatibility with run_rec which tries to store ag
draws.states = nan(Test+prior.H, 1, ndraws);
draws.obs = nan(prior.R+1+prior.H, 1, ndraws);
draws.q_mean = nan(prior.H+1, ndraws);
draws.q_var = nan(prior.H+1, ndraws);
draws.q_dens = nan(prior.H+1, size(true1,2), ndraws);

if prior.sv
    % priors
    prior.h.Vh = 10; prior.h.b0 = 0; prior.h.Vh0 = 10; prior.h.Vomegah = .2;
    prior.g.Vh = 10; prior.g.b0 = 0; prior.g.Vh0 = 10; prior.g.Vomegah = .2;
    
    % initialize the Markov chain
    h0 = log(var(y))/2;
    g0 = log(var(y))/2;
    omegah = sqrt(.2);
    omegag = sqrt(.2);
    htilde = zeros(T,1);
    gtilde = zeros(T,1);
    hh = h0 + omegah*htilde;
    gg = g0 + omegag*gtilde;
    
    % define a few things
    npts = 500; % number of grid points
    draws.omegah.grid = linspace(-1,1,npts)';
    draws.omegag.grid = linspace(-1,1,npts)';
    
    % initialize for storage
    draws.theta = nan(ndraws,4); % [omegah2 omegag2 h0 g0]
    draws.h = nan(ndraws,Test+prior.H);
    draws.g = nan(ndraws,Test+prior.H);
    draws.omegah.dens = zeros(npts,1);
    draws.omegag.dens = zeros(npts,1);
    draws.lpden = nan(ndraws,3); % log posterior densities of omegah = valh; omegag = valg; omegah = omegag
end


% Gibbs sampler
it_save = 0;
if gssettings.waitbar, waitbar_handle = waitbar(0,'Gibbs sampler','Name','Gibbs sampler running'); end
timing_start = now;
disp(['start: ',datestr(timing_start,0), '; total iterations: ', num2str(nalldraws)])
for it = 1:nalldraws
    
    % sample tau
    HinvStauH = H'*sparse(1:T,1:T,[1/Vtau*exp(-gg(1)); exp(-gg(2:end))])*H;
    invDtau =  HinvStauH + sparse(1:T,1:T,1./exp(hh));
    alptau = H\sparse(1,1,tau0,T,1);
    tauhat = invDtau\(HinvStauH*alptau + y./exp(hh));
    tau = tauhat + chol(invDtau,'lower')'\randn(T,1);
    
    % sample htilde
    Ystar = log((y-tau).^2 + .0001);
    [htilde, h0, omegah, omegahhat,Domegah] = SVRW_gam(Ystar,htilde,h0,omegah,prior.h.b0,prior.h.Vh0,prior.h.Vh,prior.h.Vomegah);
    hh = h0 + omegah*htilde;
    
    % sample g
    Ystar = log([(tau(1)-tau0)/sqrt(Vtau);tau(2:end)-tau(1:end-1)].^2 + .0001);
    [gtilde, g0, omegag, omegaghat, Domegag] = SVRW_gam(Ystar,gtilde,g0,omegag,prior.g.b0,prior.g.Vh0,prior.g.Vh,prior.g.Vomegah);
    gg = g0 + omegag*gtilde;
    
    % report progress
    if gssettings.waitbar && ~rem(it,gssettings.saveevery)
        waitbar(it/nalldraws, waitbar_handle, timing_message(it, nalldraws, timing_start))
    end
    % save current iteration if appropriate
    if it>gssettings.burnin && ~rem(it,gssettings.saveevery)
        it_save = it_save + 1;
        if prior.sv
            draws.g(it_save,1:T) = gg';
            draws.h(it_save,1:T) = hh';
            draws.theta(it_save,:) = [omegah^2 omegag^2 h0 g0];
            lh0 = -.5*log(2*pi*Domegah) - .5*(omegahhat-0)^2/Domegah;
            lg0 = -.5*log(2*pi*Domegag) - .5*(omegaghat-0)^2/Domegag;
            lhg0 = lh0 + lg0;
            draws.lpden(it_save,:) = [lh0 lg0 lhg0];
            draws.omegah.dens = draws.omegah.dens + normpdf(draws.omegah.grid,omegahhat,sqrt(Domegah));
            draws.omegag.dens = draws.omegag.dens + normpdf(draws.omegag.grid,omegaghat,sqrt(Domegag));
        end
        
        if prior.H % compute a forecast
            % compute a draw of the observables in S,...,S+H
            gtilSH = gtilde(end) + cumsum(randn(prior.H+1,1));
            htilSH = htilde(end) + cumsum(randn(prior.H+1,1));
            ggSH = g0 + omegag*gtilSH;
            hhSH = h0 + omegah*htilSH;
            draws.g(it_save,Test:end) = ggSH';
            draws.h(it_save,Test:end) = hhSH';
            sigySH = exp(0.5*hhSH);
           
            sigtauSH = exp(0.5*ggSH);
            
            tauSH = tau(end) + cumsum(sigtauSH.*randn(prior.H+1,1));
            ySH = tauSH + sigySH.*randn(prior.H+1,1);
            
            draws.obs(:,1,it_save) = [y(end-prior.R+1:end,:); ySH];
            draws.states(:,:,it_save) = [tau; tauSH];
            
            % Now compute the mean and variance of q,
            % defined as the average inflation in periods S,S+1,...,S+H
            
            % distribution of future states
            ctil = [tau(end,:); zeros(prior.H,1)];
            Tautil = blktridiag(1, -1, 0, prior.H+1);
            Rtil = diag(sigtauSH);
            mualf = Tautil\ctil;
            Valf = Tautil\Rtil; Valf = Valf*Valf';
            % distribution of future observable no. 1
            Ztil = eye(prior.H+1);
            Wtil = diag(sigySH);
            mupi = Ztil*mualf; Vpi = Ztil*Valf*Ztil' + Wtil*Wtil';
            % distribution of q
            muq = A*mupi; Vq = A*Vpi*A'; vq = diag(Vq);
            % store
            draws.q_mean(:,it_save) = muq;
            draws.q_var(:,it_save) = vq;
            for i = 1:size(trueq,2)
                draws.q_dens(:,i,it_save) = ((6.2832*vq).^-0.5).*exp(-0.5*((trueq(:,i) - muq).^2)./vq);
            end
        end
    end
end
if gssettings.waitbar, close(waitbar_handle), end
disp(timing_message(it, nalldraws, timing_start))
disp('finished Gibbs sampling')

if prior.sv
    maxlpden = max(draws.lpden);
    lpostden = log(mean(exp(draws.lpden-repmat(maxlpden,ndraws,1)))) + maxlpden;
    lpriden = [log(normpdf(0,0,sqrt(prior.h.Vomegah))) log(normpdf(0,0,sqrt(prior.g.Vomegah)))];
    lpriden(3) = sum(lpriden(1:2));
    draws.lBF = lpriden-lpostden;
    % report log BFs
    fprintf('\n');
    fprintf('log Bayes Factor\n');
    fprintf('log BF_uh       | %.1f \n', draws.lBF(1));
    fprintf('log BF_ug       | %.1f \n', draws.lBF(2));
    fprintf('log BF_u,gh     | %.1f \n', draws.lBF(3));
end

if prior.H
    draws.A = A;
    draws.q_true = trueq;
    in.cnames = strvcat(['h'; strseq('trueq',1:size(trueq,2))]);
    in.fmt = strvcat(['%12d'; repmat({'%10.4f'},size(trueq,2),1)]);
    % compute point forecasts (posterior median of the predictive density)
    draws.q_med = A*median(draws.obs(prior.R+1:end,1,:),3); % median is not a linear operator so this is wrong, but the error is small
    draws.q_err = trueq - draws.q_med*ones(1,size(trueq,2));
    disp('point forecast errors: trueq - median forecast of q')
    mprint([(0:prior.H)' draws.q_err],in)
    % compute the logscore
    draws.q_logscores = log(mean(draws.q_dens,3));
    disp('log predictive scores')
    mprint([(0:prior.H)' draws.q_logscores],in);
    
    % This is a check: compute the logscore from draws.q_mean and draws.obsvar
    % The result should be the same as draws.logscore, but this example shows how
    % to compute the logscore for some other trueq if needed.
    for i = 1:size(trueq,2)
        temp = (draws.q_mean - repmat(trueq(:,i),1,ndraws)).^2;
        temp = exp(-0.5*temp./draws.q_var);
        temp = (6.2832*draws.q_var).^(-0.5).*temp;
        temp = mean(temp,2); % average over ndraws gaussian densities
        draws.q_logscores1(:,i) = log(temp);
    end
end


end

% This function draws the log volatilities in an unobserved components
% model with a gamma prior on the error variance.
%
% See:

% Chan, J.C.C. (2016). Specification Tests for Time-Varying Parameter
% Models with Stochastic Volatility, Econometric Reviews, forthcoming

% Inputs: b0 and Vh0 are the prior mean and variance of h0;
%         Vh is the initial variance of h_1
%         Vomegah is the prior variance of omegah
function [htilde, h0, omegah, omegahhat, Domegah] = SVRW_gam(Ystar,htilde,h0,omegah,b0,Vh0,Vh,Vomegah)

T = length(htilde);
% normal mixture
pj = [0.0073 .10556 .00002 .04395 .34001 .24566 .2575];
mj = [-10.12999 -3.97281 -8.56686 2.77786 .61942 1.79518 -1.08819] - 1.2704;  %% means already adjusted!! %%
sigj = [5.79596 2.61369 5.17950 .16735 .64009 .34023 1.26261];
sqrtsigj = sqrt(sigj);

% sample S from a 7-point distrete distribution
temprand = rand(T,1);
q = repmat(pj,T,1).*normpdf(repmat(Ystar,1,7),repmat(h0+omegah*htilde,1,7)+repmat(mj,T,1), repmat(sqrtsigj,T,1));
q = q./repmat(sum(q,2),1,7);
S = 7 - sum(repmat(temprand,1,7)<cumsum(q,2),2)+1;

% sample htilde
% y^* = h0 + omegah htilde + d + \epsilon, \epsilon \sim N(0,\Omega),
% H htilde = \nu, \nu \ sim N(0,S),
% where d_t = Ez_t, \Omega = diag(\omega_1,\ldots,\omega_n),
% \omega_t = var z_t, S = diag(Vh, 1, \ldots, 1)
Hh = speye(T) - sparse(2:T,1:(T-1),ones(1,T-1),T,T);
invSh = sparse(1:T,1:T,[1/Vh; ones(T-1,1)]);
dconst = mj(S)'; invOmega = sparse(1:T,1:T,1./sigj(S));
Kh = Hh'*invSh*Hh + invOmega*omegah^2;
htildehat = Kh\(invOmega*omegah*(Ystar-dconst-h0));
htilde = htildehat + chol(Kh,'lower')'\randn(T,1);

% sample h0 and omegah
Xbeta = [ones(T,1) htilde];
invVbeta = diag([1/Vh0 1/Vomegah]);
XbetainvOmega = Xbeta'*invOmega;
invDbeta = invVbeta + XbetainvOmega*Xbeta;
betahat = invDbeta\(invVbeta*[b0;0] + XbetainvOmega*(Ystar-dconst));
beta = betahat + chol(invDbeta,'lower')'\randn(2,1);
h0 = beta(1); omegah = beta(2);

U = -1 + 2*(rand>0.5);
htilde = U*htilde;
omegah = U*omegah;

% compute the mean and variance of the conditional posterior of omegah
Xbeta = [ones(T,1) htilde];
Dbeta = (invVbeta + Xbeta'*invOmega*Xbeta)\speye(2);
omegahhat = betahat(2);
Domegah = Dbeta(2,2);
end